/*
    File:               argonMsimg32Dll.cpp
    Description:        Extensions and emulations of parts of the
                        system's MSIMG32.DLL
                        useful mainly to support system features that
                        are not or may be not present on all available
                        operating system versions out there.
                        This file is part of the ArgonLibrary package.

    Author:             Roman Komary
    created on:         10.11.2003
    last updated:       03.02.2006 (Roman Komary)

    Copyright (c) 2003-2006, Roman Komary.

    Distributed under the Boost Software License, Version 1.0. (See
    accompanying file LICENSE_1_0.ArgonLibrary.txt or copy at
    http://www.boost.org/LICENSE_1_0.txt)
*/

#define __argonMsimg32Dll_cpp
#define __an_ArgonLibrary_cpp       // helpful flag, if you need to make conditional compilations e.g. in "stdafx.h"

#include "stdafx.h"
#include "argonMsimg32Dll.h"

#ifndef _argon_no_includes
    #include "argonLocking.h"
#endif

namespace ArgonLibrary
{




BOOL WINAPI argonAlphaBlend(
    HDC hdcDest,                 // handle to destination DC
    int nXOriginDest,            // x-coord of upper-left corner
    int nYOriginDest,            // y-coord of upper-left corner
    int nWidthDest,              // destination width
    int nHeightDest,             // destination height
    HDC hdcSrc,                  // handle to source DC
    int nXOriginSrc,             // x-coord of upper-left corner
    int nYOriginSrc,             // y-coord of upper-left corner
    int nWidthSrc,               // source width
    int nHeightSrc,              // source height
    BLENDFUNCTION blendFunction  // alpha-blending function
)
{
    Msimg32Dll::LoadAPI();
    if(   !Msimg32Dll::s_pfnAlphaBlend ) { SetLastError(ERROR_CALL_NOT_IMPLEMENTED); return FALSE; }
    return Msimg32Dll::s_pfnAlphaBlend( hdcDest, nXOriginDest, nYOriginDest, nWidthDest, nHeightDest,
                                        hdcSrc,  nXOriginSrc,  nYOriginSrc,  nWidthSrc,  nHeightSrc,
                                        blendFunction );
}

BOOL WINAPI argonTransparentBlt(
    HDC hdcDest,                 // handle to destination DC
    int nXOriginDest,            // x-coord of destination upper-left corner
    int nYOriginDest,            // y-coord of destination upper-left corner
    int nWidthDest,              // width of destination rectangle
    int nHeightDest,             // height of destination rectangle
    HDC hdcSrc,                  // handle to source DC
    int nXOriginSrc,             // x-coord of source upper-left corner
    int nYOriginSrc,             // y-coord of source upper-left corner
    int nWidthSrc,               // width of source rectangle
    int nHeightSrc,              // height of source rectangle
    UINT crTransparent           // color to make transparent
)
{
    Msimg32Dll::LoadAPI();
    if(   !Msimg32Dll::s_pfnTransparentBlt ) { SetLastError(ERROR_CALL_NOT_IMPLEMENTED); return FALSE; }
    return Msimg32Dll::s_pfnTransparentBlt( hdcDest, nXOriginDest, nYOriginDest, nWidthDest, nHeightDest,
                                            hdcSrc,  nXOriginSrc,  nYOriginSrc,  nWidthSrc,  nHeightSrc,
                                            crTransparent );
}

BOOL WINAPI argonGradientFill(
    HDC hdc,                     // handle to DC
    PTRIVERTEX pVertex,          // array of vertices
    ULONG dwNumVertex,           // number of vertices
    PVOID pMesh,                 // array of gradients
    ULONG dwNumMesh,             // size of gradient array
    ULONG dwMode                 // gradient fill mode
)
{
    Msimg32Dll::LoadAPI();
    if(   !Msimg32Dll::s_pfnGradientFill ) { SetLastError(ERROR_CALL_NOT_IMPLEMENTED); return FALSE; }
    return Msimg32Dll::s_pfnGradientFill( hdc, pVertex, dwNumVertex,
                                          pMesh, dwNumMesh, dwMode );
}



BOOL GradientFillRect( HDC hdc, const RECT *pRect, COLORREF cr1, COLORREF cr2, BOOL bHorz, BOOL bWithAlpha )
{
    if( !hdc || !pRect ) { SetLastError(ERROR_INVALID_PARAMETER); return FALSE; }
    if( ::IsRectEmpty( pRect ) ) { SetLastError(ERROR_SUCCESS); return TRUE; }

    TRIVERTEX tvx[2];
    tvx[0].x = pRect->left;
    tvx[0].y = pRect->top;
    tvx[1].x = pRect->right;
    tvx[1].y = pRect->bottom;
    tvx[0].Red   = ((COLOR16)GetRValue(cr1)) << 8;
    tvx[0].Green = ((COLOR16)GetGValue(cr1)) << 8;
    tvx[0].Blue  = ((COLOR16)GetBValue(cr1)) << 8;
    if( bWithAlpha ) tvx[0].Alpha = ((COLOR16)GetAValue(cr1)) << 8;
    else             tvx[0].Alpha = 0;
    tvx[1].Red   = ((COLOR16)GetRValue(cr2)) << 8;
    tvx[1].Green = ((COLOR16)GetGValue(cr2)) << 8;
    tvx[1].Blue  = ((COLOR16)GetBValue(cr2)) << 8;
    if( bWithAlpha ) tvx[1].Alpha = ((COLOR16)GetAValue(cr2)) << 8;
    else             tvx[1].Alpha = 0;

    GRADIENT_RECT gr;
    gr.UpperLeft  = 0;
    gr.LowerRight = 1;

    return GradientFill( hdc, tvx, 2, &gr, 1, bHorz ? GRADIENT_FILL_RECT_H : GRADIENT_FILL_RECT_V );
}




Msimg32Dll::Unloader    Msimg32Dll::s_Unloader;
HMODULE                 Msimg32Dll::s_hModule           = NULL;
TfnAlphaBlend           Msimg32Dll::s_pfnAlphaBlend     = NULL;
TfnTransparentBlt       Msimg32Dll::s_pfnTransparentBlt = NULL;
TfnGradientFill         Msimg32Dll::s_pfnGradientFill   = NULL;
bool                    Msimg32Dll::s_bAutoUnload       = true;

Msimg32Dll::Msimg32Dll()
{
}

// Remarks: We don't need any object of #Msimg32Dll#.
//  We just need its static members.



bool Msimg32Dll::IsAlphaBlendSupported()
{
    LoadAPI();
    return s_pfnAlphaBlend != NULL;
}

bool Msimg32Dll::IsGradientFillSupported()
{
    LoadAPI();
    return s_pfnGradientFill != NULL;
}



void Msimg32Dll::LoadAPI()
{
    if( s_hModule ) return;

    LockModuleWide lmw;

    if( s_hModule ) return;

    Nullify();

    s_hModule = ::LoadLibrary( _T(MSIMG32DLL_NAME) );

    if( s_hModule )
    {
        s_pfnAlphaBlend     = (TfnAlphaBlend    ) GetProcAddress( s_hModule, "AlphaBlend"     );
        s_pfnTransparentBlt = (TfnTransparentBlt) GetProcAddress( s_hModule, "TransparentBlt" );
        s_pfnGradientFill   = (TfnGradientFill  ) GetProcAddress( s_hModule, "GradientFill"   );
    }
}

void Msimg32Dll::UnloadAPI()
{
    LockModuleWide lmw;

    if( s_hModule )
        ::FreeLibrary( s_hModule );

    Nullify();
}



void Msimg32Dll::Nullify()
{
    s_pfnAlphaBlend     = NULL;
    s_pfnTransparentBlt = NULL;
    s_pfnGradientFill   = NULL;
    s_hModule = NULL;
}




}
