/*
    File:               argonMsimg32Dll.h
    Description:        Extensions and emulations of parts of the
                        system's MSIMG32.DLL
                        useful mainly to support system features that
                        are not or may be not present on all available
                        operating system versions out there.
                        This file is part of the ArgonLibrary package.

    Author:             Roman Komary
    created on:         10.11.2003
    last updated:       03.02.2006 (Roman Komary)

    Copyright (c) 2003-2006, Roman Komary.

    Distributed under the Boost Software License, Version 1.0. (See
    accompanying file LICENSE_1_0.ArgonLibrary.txt or copy at
    http://www.boost.org/LICENSE_1_0.txt)
*/

#ifndef __argonMsimg32Dll_h
#define __argonMsimg32Dll_h

#ifndef __ArgonLibrary__
#define __ArgonLibrary__            // helpful flag, if you need to make conditional compilations somewhere in your sourcecode
#endif

namespace ArgonLibrary
{




/*
    -----------------------------------------------------------------
    Usage:

    Simply call the functions #AlphaBlend()#, #TransparentBlt()#,
    #GradientFill()# etc. in your source code as you need it.

    All these functions defined in here are emulating the system behavior
    that way, that they load the functions during runtime from the system
    and only if this did succeed, they will be called.

    The first time you call any of these functions, the library will try
    to be loaded and the function pointers will try to be retrieved.

    The unloading of the library will be done automatically
    during the deleting of the global variables.
    If you included this file in a DLL project, you will not want this
    behavior to happen because it should be prevented to call FreeLibrary()
    when a DLL is detached from a process (where global variables will be
    deleted).
    So in the case of DLL project, it is recommended to export an own
    DLL-deinit-function which you should redirect to a call to
    #Msimg32Dll::UnloadAPI()#. If the user then calls this deinit-function,
    the MSIMG32.DLL will be unloaded and later on, when your DLL is detachted
    from a process, it is harmless to call #Msimg32Dll::UnloadAPI()# again
    because it will not do anything any more.

    -----------------------------------------------------------------
    Additional, new functions:

    GradientFillRect
*/


#define MSIMG32DLL_NAME             "MSIMG32.DLL"


// ____ Graphic features __________________________________
//      AlphaBlend
//      GradientFill

#ifndef GetAValue
#define GetAValue(rgba)             ((BYTE)((rgba)>>24))
#endif

#ifndef AC_SRC_ALPHA
#define AC_SRC_ALPHA                0x00000001
#endif

#ifndef AC_SRC_OVER
#define AC_SRC_OVER                 0x00
#define AC_SRC_NO_PREMULT_ALPHA     0x01
#define AC_SRC_NO_ALPHA             0x02
#define AC_DST_NO_PREMULT_ALPHA     0x10
#define AC_DST_NO_ALPHA             0x20
#endif

#ifndef GRADIENT_FILL_RECT_H
#define GRADIENT_FILL_RECT_H        0x00000000
#define GRADIENT_FILL_RECT_V        0x00000001
#define GRADIENT_FILL_TRIANGLE      0x00000002
#define GRADIENT_FILL_OP_FLAG       0x000000ff
#endif

/*
typedef USHORT COLOR16;

typedef struct _TRIVERTEX
{
    LONG    x;
    LONG    y;
    COLOR16 Red;
    COLOR16 Green;
    COLOR16 Blue;
    COLOR16 Alpha;
}TRIVERTEX,*PTRIVERTEX,*LPTRIVERTEX;

typedef struct _GRADIENT_TRIANGLE
{
    ULONG Vertex1;
    ULONG Vertex2;
    ULONG Vertex3;
} GRADIENT_TRIANGLE,*PGRADIENT_TRIANGLE,*LPGRADIENT_TRIANGLE;

typedef struct _GRADIENT_RECT
{
    ULONG UpperLeft;
    ULONG LowerRight;
}GRADIENT_RECT,*PGRADIENT_RECT,*LPGRADIENT_RECT;

typedef struct _BLENDFUNCTION
{
    BYTE   BlendOp;
    BYTE   BlendFlags;
    BYTE   SourceConstantAlpha;
    BYTE   AlphaFormat;
}BLENDFUNCTION,*PBLENDFUNCTION;
*/

#if defined (__cplusplus)
extern "C" {
#endif

typedef BOOL (WINAPI *TfnAlphaBlend)(
    HDC hdcDest,                 // handle to destination DC
    int nXOriginDest,            // x-coord of upper-left corner
    int nYOriginDest,            // y-coord of upper-left corner
    int nWidthDest,              // destination width
    int nHeightDest,             // destination height
    HDC hdcSrc,                  // handle to source DC
    int nXOriginSrc,             // x-coord of upper-left corner
    int nYOriginSrc,             // y-coord of upper-left corner
    int nWidthSrc,               // source width
    int nHeightSrc,              // source height
    BLENDFUNCTION blendFunction  // alpha-blending function
);

BOOL WINAPI argonAlphaBlend(
    HDC hdcDest,                 // handle to destination DC
    int nXOriginDest,            // x-coord of upper-left corner
    int nYOriginDest,            // y-coord of upper-left corner
    int nWidthDest,              // destination width
    int nHeightDest,             // destination height
    HDC hdcSrc,                  // handle to source DC
    int nXOriginSrc,             // x-coord of upper-left corner
    int nYOriginSrc,             // y-coord of upper-left corner
    int nWidthSrc,               // source width
    int nHeightSrc,              // source height
    BLENDFUNCTION blendFunction  // alpha-blending function
);

typedef BOOL (WINAPI *TfnTransparentBlt)(
    HDC hdcDest,                 // handle to destination DC
    int nXOriginDest,            // x-coord of destination upper-left corner
    int nYOriginDest,            // y-coord of destination upper-left corner
    int nWidthDest,              // width of destination rectangle
    int nHeightDest,             // height of destination rectangle
    HDC hdcSrc,                  // handle to source DC
    int nXOriginSrc,             // x-coord of source upper-left corner
    int nYOriginSrc,             // y-coord of source upper-left corner
    int nWidthSrc,               // width of source rectangle
    int nHeightSrc,              // height of source rectangle
    UINT crTransparent           // color to make transparent
);

BOOL WINAPI argonTransparentBlt(
    HDC hdcDest,                 // handle to destination DC
    int nXOriginDest,            // x-coord of destination upper-left corner
    int nYOriginDest,            // y-coord of destination upper-left corner
    int nWidthDest,              // width of destination rectangle
    int nHeightDest,             // height of destination rectangle
    HDC hdcSrc,                  // handle to source DC
    int nXOriginSrc,             // x-coord of source upper-left corner
    int nYOriginSrc,             // y-coord of source upper-left corner
    int nWidthSrc,               // width of source rectangle
    int nHeightSrc,              // height of source rectangle
    UINT crTransparent           // color to make transparent
);

typedef BOOL (WINAPI *TfnGradientFill)(
    HDC hdc,                     // handle to DC
    PTRIVERTEX pVertex,          // array of vertices
    ULONG dwNumVertex,           // number of vertices
    PVOID pMesh,                 // array of gradients
    ULONG dwNumMesh,             // size of gradient array
    ULONG dwMode                 // gradient fill mode
);

BOOL WINAPI argonGradientFill(
    HDC hdc,                     // handle to DC
    PTRIVERTEX pVertex,          // array of vertices
    ULONG dwNumVertex,           // number of vertices
    PVOID pMesh,                 // array of gradients
    ULONG dwNumMesh,             // size of gradient array
    ULONG dwMode                 // gradient fill mode
);

#define AlphaBlend              ArgonLibrary::argonAlphaBlend
#define TransparentBlt          ArgonLibrary::argonTransparentBlt
#define GradientFill            ArgonLibrary::argonGradientFill

// additional, new functions

///<param name="bHorz"> TRUE = horizontal, otherwise vertical </param>
BOOL GradientFillRect( HDC hdc, const RECT *pRect, COLORREF cr1, COLORREF cr2, BOOL bHorz, BOOL bWithAlpha );

#if defined (__cplusplus)
}
#endif


class Msimg32Dll
{
    protected :
        class Unloader
        {
            public :
                Unloader() {  }
                ~Unloader() { if( Msimg32Dll::s_bAutoUnload ) Msimg32Dll::UnloadAPI(); }
        };

        static Unloader s_Unloader;

    public :
        static HMODULE s_hModule;
        static TfnAlphaBlend     s_pfnAlphaBlend;
        static TfnTransparentBlt s_pfnTransparentBlt;
        static TfnGradientFill   s_pfnGradientFill;
        static bool s_bAutoUnload;  // true = automatically unloads the Dll when global objects are destructed.

        Msimg32Dll();

        static bool IsAlphaBlendSupported();
        static bool IsGradientFillSupported();

        static void LoadAPI();
        static void UnloadAPI();

    protected :
        static void Nullify();
};




}

#endif
