/*
    File:               argonProcessesModules.cpp
    Description:        Enumeration Processes and its Modules
                        is easily possible using these classes.
                        It will at runtime automtaically use
                        ToolHelp-Api or PSAPI depending on what
                        is present on the current system.
                        This file is part of the ArgonLibrary package.

    Author:             Roman Komary
    created on:         25.11.2005
    last updated:       02.02.2006 (Roman Komary)

    Copyright (c) 2005-2006, Roman Komary.

    Distributed under the Boost Software License, Version 1.0. (See
    accompanying file LICENSE_1_0.ArgonLibrary.txt or copy at
    http://www.boost.org/LICENSE_1_0.txt)
*/

#define __argonProcessesModules_cpp
#define __an_ArgonLibrary_cpp       // helpful flag, if you need to make conditional compilations e.g. in "stdafx.h"

#include "stdafx.h"
#include "argonProcessesModules.h"

#ifndef _argon_no_includes
    #include "argonLocking.h"
#endif

namespace ArgonLibrary
{




#ifndef _argon_no_psapi

BOOL WINAPI argonEnumProcesses(
    DWORD * lpidProcess,
    DWORD   cb,
    DWORD * cbNeeded
)
{
    PsApiDll::LoadAPI();
    if(   !PsApiDll::s_pfnEnumProcesses ) { SetLastError(ERROR_CALL_NOT_IMPLEMENTED); return FALSE; }
    return PsApiDll::s_pfnEnumProcesses( lpidProcess, cb, cbNeeded );
}

BOOL WINAPI argonEnumProcessModules(
    HANDLE hProcess,
    HMODULE *lphModule,
    DWORD cb,
    LPDWORD lpcbNeeded
)
{
    PsApiDll::LoadAPI();
    if(   !PsApiDll::s_pfnEnumProcessModules ) { SetLastError(ERROR_CALL_NOT_IMPLEMENTED); return FALSE; }
    return PsApiDll::s_pfnEnumProcessModules( hProcess, lphModule, cb, lpcbNeeded );
}

DWORD WINAPI argonGetModuleBaseNameA(
    HANDLE hProcess,
    HMODULE hModule,
    LPSTR lpBaseName,
    DWORD nSize
)
{
    PsApiDll::LoadAPI();
    if(   !PsApiDll::s_pfnGetModuleBaseNameA ) { SetLastError(ERROR_CALL_NOT_IMPLEMENTED); return 0; }
    return PsApiDll::s_pfnGetModuleBaseNameA( hProcess, hModule, lpBaseName, nSize );
}

DWORD WINAPI argonGetModuleBaseNameW(
    HANDLE hProcess,
    HMODULE hModule,
    LPWSTR lpBaseName,
    DWORD nSize
)
{
    PsApiDll::LoadAPI();
    if(   !PsApiDll::s_pfnGetModuleBaseNameW ) { SetLastError(ERROR_CALL_NOT_IMPLEMENTED); return 0; }
    return PsApiDll::s_pfnGetModuleBaseNameW( hProcess, hModule, lpBaseName, nSize );
}

DWORD WINAPI argonGetModuleFileNameExA(
    HANDLE hProcess,
    HMODULE hModule,
    LPSTR lpFilename,
    DWORD nSize
)
{
    PsApiDll::LoadAPI();
    if(   !PsApiDll::s_pfnGetModuleFileNameExA ) { SetLastError(ERROR_CALL_NOT_IMPLEMENTED); return 0; }
    return PsApiDll::s_pfnGetModuleFileNameExA( hProcess, hModule, lpFilename, nSize );
}

DWORD WINAPI argonGetModuleFileNameExW(
    HANDLE hProcess,
    HMODULE hModule,
    LPWSTR lpFilename,
    DWORD nSize
)
{
    PsApiDll::LoadAPI();
    if(   !PsApiDll::s_pfnGetModuleFileNameExW ) { SetLastError(ERROR_CALL_NOT_IMPLEMENTED); return 0; }
    return PsApiDll::s_pfnGetModuleFileNameExW( hProcess, hModule, lpFilename, nSize );
}

BOOL WINAPI argonGetModuleInformation(
    HANDLE hProcess,
    HMODULE hModule,
    LPMODULEINFO lpmodinfo,
    DWORD cb
)
{
    PsApiDll::LoadAPI();
    if(   !PsApiDll::s_pfnGetModuleInformation ) { SetLastError(ERROR_CALL_NOT_IMPLEMENTED); return FALSE; }
    return PsApiDll::s_pfnGetModuleInformation( hProcess, hModule, lpmodinfo, cb );
}

BOOL WINAPI argonGetProcessMemoryInfo(
    HANDLE Process,
    PPROCESS_MEMORY_COUNTERS ppsmemCounters,
    DWORD cb
)
{
    PsApiDll::LoadAPI();
    if(   !PsApiDll::s_pfnGetProcessMemoryInfo ) { SetLastError(ERROR_CALL_NOT_IMPLEMENTED); return FALSE; }
    return PsApiDll::s_pfnGetProcessMemoryInfo( Process, ppsmemCounters, cb );
}

BOOL WINAPI argonGetPerformanceInfo(
    PPERFORMACE_INFORMATION pPerformanceInformation,
    DWORD cb
)
{
    PsApiDll::LoadAPI();
    if(   !PsApiDll::s_pfnGetPerformanceInfo ) { SetLastError(ERROR_CALL_NOT_IMPLEMENTED); return FALSE; }
    return PsApiDll::s_pfnGetPerformanceInfo( pPerformanceInformation, cb );
}

DWORD WINAPI argonGetProcessImageFileNameA(
    HANDLE hProcess,
    LPSTR lpImageFileName,
    DWORD nSize
)
{
    PsApiDll::LoadAPI();
    if(   !PsApiDll::s_pfnGetProcessImageFileNameA ) { SetLastError(ERROR_CALL_NOT_IMPLEMENTED); return 0; }
    return PsApiDll::s_pfnGetProcessImageFileNameA( hProcess, lpImageFileName, nSize );
}

DWORD WINAPI argonGetProcessImageFileNameW(
    HANDLE hProcess,
    LPWSTR lpImageFileName,
    DWORD nSize
)
{
    PsApiDll::LoadAPI();
    if(   !PsApiDll::s_pfnGetProcessImageFileNameW ) { SetLastError(ERROR_CALL_NOT_IMPLEMENTED); return 0; }
    return PsApiDll::s_pfnGetProcessImageFileNameW( hProcess, lpImageFileName, nSize );
}



String GetModuleFileNameExStr( HANDLE hProcess, HMODULE hModule )
{
    TCHAR s[_MAX_PATH_ + 1];
    // #::GetModuleFileNameEx()# will *not* append a null-terminator
    // if the passed string buffer is too short (otherwise it will).
    if( 0 == argonGetModuleFileNameEx( hProcess, hModule, s, _countof(s) ) )
        { SetLastError(ERROR_CALL_NOT_IMPLEMENTED); return String(); }
    s[_MAX_PATH_] = (TCHAR) 0;  // this *is* necessary
    // As I figured out, the returned path could be not case-correct
    // for case-insensitive file systems on some operating systems.
    // On WindowsXP, it *is* case-correct.

    _ospath_( s );

    SetLastError( ERROR_SUCCESS );

    return s;
}




PsApiDll::Unloader              PsApiDll::s_Unloader;
HMODULE                         PsApiDll::s_hModule                     = NULL;
TfnEnumProcesses                PsApiDll::s_pfnEnumProcesses            = NULL;
TfnEnumProcessModules           PsApiDll::s_pfnEnumProcessModules       = NULL;
TfnGetModuleBaseNameA           PsApiDll::s_pfnGetModuleBaseNameA       = NULL;
TfnGetModuleBaseNameW           PsApiDll::s_pfnGetModuleBaseNameW       = NULL;
TfnGetModuleFileNameExA         PsApiDll::s_pfnGetModuleFileNameExA     = NULL;
TfnGetModuleFileNameExW         PsApiDll::s_pfnGetModuleFileNameExW     = NULL;
TfnGetModuleInformation         PsApiDll::s_pfnGetModuleInformation     = NULL;
TfnGetProcessMemoryInfo         PsApiDll::s_pfnGetProcessMemoryInfo     = NULL;
TfnGetPerformanceInfo           PsApiDll::s_pfnGetPerformanceInfo       = NULL;
TfnGetProcessImageFileNameA     PsApiDll::s_pfnGetProcessImageFileNameA = NULL;
TfnGetProcessImageFileNameW     PsApiDll::s_pfnGetProcessImageFileNameW = NULL;
bool                            PsApiDll::s_bAutoUnload                 = true;

PsApiDll::PsApiDll()
{
}

// Remarks: We don't need any object of #PsApiDll#.
//  We just need its static members.



bool PsApiDll::PsApiExists()
{
    LoadAPI();
    return s_hModule != NULL;
}

///<remarks>
///  <b>GetPerformanceInfo()</b> and e.g. <b>GetProcessImageFileName()</b>
///  are available only beginning with Windows XP or Windows Server 2003.
///  So we make an explicit check on this in here.
///  Ok, you could check member <b>s_pfnGetPerformanceInfo</b> by hand,
///  but seeing this method here will remaind you better of the fact,
///  that earlier Windows versions do not include this function.
///</remarks>
bool PsApiDll::hasGetPerformanceInfo()
{
    LoadAPI();
    return s_pfnGetPerformanceInfo != NULL;
}



void PsApiDll::LoadAPI()
{
    if( s_hModule ) return;

    LockModuleWide lmw;

    if( s_hModule ) return;

    Nullify();

    s_hModule = ::LoadLibrary( _T(PSAPIDLL_NAME) );

    if( s_hModule )
    {
        s_pfnEnumProcesses            = (TfnEnumProcesses           ) GetProcAddress( s_hModule, "EnumProcesses"            );
        s_pfnEnumProcessModules       = (TfnEnumProcessModules      ) GetProcAddress( s_hModule, "EnumProcessModules"       );
        s_pfnGetModuleBaseNameA       = (TfnGetModuleBaseNameA      ) GetProcAddress( s_hModule, "GetModuleBaseNameA"       );
        s_pfnGetModuleBaseNameW       = (TfnGetModuleBaseNameW      ) GetProcAddress( s_hModule, "GetModuleBaseNameW"       );
        s_pfnGetModuleFileNameExA     = (TfnGetModuleFileNameExA    ) GetProcAddress( s_hModule, "GetModuleFileNameExA"     );
        s_pfnGetModuleFileNameExW     = (TfnGetModuleFileNameExW    ) GetProcAddress( s_hModule, "GetModuleFileNameExW"     );
        s_pfnGetModuleInformation     = (TfnGetModuleInformation    ) GetProcAddress( s_hModule, "GetModuleInformation"     );
        s_pfnGetProcessMemoryInfo     = (TfnGetProcessMemoryInfo    ) GetProcAddress( s_hModule, "GetProcessMemoryInfo"     );
        s_pfnGetPerformanceInfo       = (TfnGetPerformanceInfo      ) GetProcAddress( s_hModule, "GetPerformanceInfo"       );
        s_pfnGetProcessImageFileNameA = (TfnGetProcessImageFileNameA) GetProcAddress( s_hModule, "GetProcessImageFileNameA" );
        s_pfnGetProcessImageFileNameW = (TfnGetProcessImageFileNameW) GetProcAddress( s_hModule, "GetProcessImageFileNameW" );
    }
}

void PsApiDll::UnloadAPI()
{
    LockModuleWide lmw;

    if( s_hModule )
        ::FreeLibrary( s_hModule );

    Nullify();
}



void PsApiDll::Nullify()
{
    s_pfnEnumProcesses            = NULL;
    s_pfnEnumProcessModules       = NULL;
    s_pfnGetModuleBaseNameA       = NULL;
    s_pfnGetModuleBaseNameW       = NULL;
    s_pfnGetModuleFileNameExA     = NULL;
    s_pfnGetModuleFileNameExW     = NULL;
    s_pfnGetModuleInformation     = NULL;
    s_pfnGetProcessMemoryInfo     = NULL;
    s_pfnGetPerformanceInfo       = NULL;
    s_pfnGetProcessImageFileNameA = NULL;
    s_pfnGetProcessImageFileNameW = NULL;
    s_hModule = NULL;
}

#endif




#ifndef _argon_no_toolhelp32api

HANDLE WINAPI argonCreateToolhelp32Snapshot(
    DWORD dwFlags,
    DWORD th32ProcessID
)
{
    ToolHelp32Api::LoadAPI();
    if(   !ToolHelp32Api::s_pfnCreateToolhelp32Snapshot ) { SetLastError(ERROR_CALL_NOT_IMPLEMENTED); return INVALID_HANDLE_VALUE; }
    return ToolHelp32Api::s_pfnCreateToolhelp32Snapshot( dwFlags, th32ProcessID );
}

BOOL WINAPI argonHeap32ListFirst(
    HANDLE hSnapshot,
    LPHEAPLIST32 lphl
)
{
    ToolHelp32Api::LoadAPI();
    if(   !ToolHelp32Api::s_pfnHeap32ListFirst ) { SetLastError(ERROR_CALL_NOT_IMPLEMENTED); return FALSE; }
    return ToolHelp32Api::s_pfnHeap32ListFirst( hSnapshot, lphl );
}

BOOL WINAPI argonHeap32ListNext(
    HANDLE hSnapshot,
    LPHEAPLIST32 lphl
)
{
    ToolHelp32Api::LoadAPI();
    if(   !ToolHelp32Api::s_pfnHeap32ListNext ) { SetLastError(ERROR_CALL_NOT_IMPLEMENTED); return FALSE; }
    return ToolHelp32Api::s_pfnHeap32ListNext( hSnapshot, lphl );
}

BOOL WINAPI argonHeap32First(
    LPHEAPENTRY32 lphe,
    DWORD th32ProcessID,
    ULONG_PTR th32HeapID
)
{
    ToolHelp32Api::LoadAPI();
    if(   !ToolHelp32Api::s_pfnHeap32First ) { SetLastError(ERROR_CALL_NOT_IMPLEMENTED); return FALSE; }
    return ToolHelp32Api::s_pfnHeap32First( lphe, th32ProcessID, th32HeapID );
}

BOOL WINAPI argonHeap32Next(
    LPHEAPENTRY32 lphe
)
{
    ToolHelp32Api::LoadAPI();
    if(   !ToolHelp32Api::s_pfnHeap32Next ) { SetLastError(ERROR_CALL_NOT_IMPLEMENTED); return FALSE; }
    return ToolHelp32Api::s_pfnHeap32Next( lphe );
}

BOOL WINAPI argonToolhelp32ReadProcessMemory(
    DWORD   th32ProcessID,
    LPCVOID lpBaseAddress,
    LPVOID  lpBuffer,
    SIZE_T  cbRead,
    SIZE_T *lpNumberOfBytesRead
)
{
    ToolHelp32Api::LoadAPI();
    if(   !ToolHelp32Api::s_pfnToolhelp32ReadProcessMemory ) { SetLastError(ERROR_CALL_NOT_IMPLEMENTED); return FALSE; }
    return ToolHelp32Api::s_pfnToolhelp32ReadProcessMemory( th32ProcessID, lpBaseAddress, lpBuffer, cbRead, lpNumberOfBytesRead );
}

BOOL WINAPI argonProcess32FirstW(
    HANDLE hSnapshot,
    LPPROCESSENTRY32W lppe
)
{
    ToolHelp32Api::LoadAPI();
    if(   !ToolHelp32Api::s_pfnProcess32FirstW ) { SetLastError(ERROR_CALL_NOT_IMPLEMENTED); return FALSE; }
    return ToolHelp32Api::s_pfnProcess32FirstW( hSnapshot, lppe );
}

BOOL WINAPI argonProcess32NextW(
    HANDLE hSnapshot,
    LPPROCESSENTRY32W lppe
)
{
    ToolHelp32Api::LoadAPI();
    if(   !ToolHelp32Api::s_pfnProcess32NextW ) { SetLastError(ERROR_CALL_NOT_IMPLEMENTED); return FALSE; }
    return ToolHelp32Api::s_pfnProcess32NextW( hSnapshot, lppe );
}

BOOL WINAPI argonProcess32First(
    HANDLE hSnapshot,
    tagPROCESSENTRY32 *lppe  // LPPROCESSENTRY32 lppe
)
{
    ToolHelp32Api::LoadAPI();
    if(   !ToolHelp32Api::s_pfnProcess32First ) { SetLastError(ERROR_CALL_NOT_IMPLEMENTED); return FALSE; }
    return ToolHelp32Api::s_pfnProcess32First( hSnapshot, lppe );
}

BOOL WINAPI argonProcess32Next(
    HANDLE hSnapshot,
    tagPROCESSENTRY32 *lppe  // LPPROCESSENTRY32 lppe
)
{
    ToolHelp32Api::LoadAPI();
    if(   !ToolHelp32Api::s_pfnProcess32Next ) { SetLastError(ERROR_CALL_NOT_IMPLEMENTED); return FALSE; }
    return ToolHelp32Api::s_pfnProcess32Next( hSnapshot, lppe );
}

BOOL WINAPI argonThread32First(
    HANDLE hSnapshot,
    LPTHREADENTRY32 lpte
)
{
    ToolHelp32Api::LoadAPI();
    if(   !ToolHelp32Api::s_pfnThread32First ) { SetLastError(ERROR_CALL_NOT_IMPLEMENTED); return FALSE; }
    return ToolHelp32Api::s_pfnThread32First( hSnapshot, lpte );
}

BOOL WINAPI argonThread32Next(
    HANDLE hSnapshot,
    LPTHREADENTRY32 lpte
)
{
    ToolHelp32Api::LoadAPI();
    if(   !ToolHelp32Api::s_pfnThread32Next ) { SetLastError(ERROR_CALL_NOT_IMPLEMENTED); return FALSE; }
    return ToolHelp32Api::s_pfnThread32Next( hSnapshot, lpte );
}

BOOL WINAPI argonModule32FirstW(
    HANDLE hSnapshot,
    LPMODULEENTRY32W lpme
)
{
    ToolHelp32Api::LoadAPI();
    if(   !ToolHelp32Api::s_pfnModule32FirstW ) { SetLastError(ERROR_CALL_NOT_IMPLEMENTED); return FALSE; }
    return ToolHelp32Api::s_pfnModule32FirstW( hSnapshot, lpme );
}

BOOL WINAPI argonModule32NextW(
    HANDLE hSnapshot,
    LPMODULEENTRY32W lpme
)
{
    ToolHelp32Api::LoadAPI();
    if(   !ToolHelp32Api::s_pfnModule32NextW ) { SetLastError(ERROR_CALL_NOT_IMPLEMENTED); return FALSE; }
    return ToolHelp32Api::s_pfnModule32NextW( hSnapshot, lpme );
}

BOOL WINAPI argonModule32First(
    HANDLE hSnapshot,
    tagMODULEENTRY32 *lpme  // LPMODULEENTRY32 lpme
)
{
    ToolHelp32Api::LoadAPI();
    if(   !ToolHelp32Api::s_pfnModule32First ) { SetLastError(ERROR_CALL_NOT_IMPLEMENTED); return FALSE; }
    return ToolHelp32Api::s_pfnModule32First( hSnapshot, lpme );
}

BOOL WINAPI argonModule32Next(
    HANDLE hSnapshot,
    tagMODULEENTRY32 *lpme  // LPMODULEENTRY32 lpme
)
{
    ToolHelp32Api::LoadAPI();
    if(   !ToolHelp32Api::s_pfnModule32Next ) { SetLastError(ERROR_CALL_NOT_IMPLEMENTED); return FALSE; }
    return ToolHelp32Api::s_pfnModule32Next( hSnapshot, lpme );
}




ToolHelp32Api::Unloader         ToolHelp32Api::s_Unloader;
HMODULE                         ToolHelp32Api::s_hModule                        = NULL;
TfnCreateToolhelp32Snapshot     ToolHelp32Api::s_pfnCreateToolhelp32Snapshot    = NULL;
TfnHeap32ListFirst              ToolHelp32Api::s_pfnHeap32ListFirst             = NULL;
TfnHeap32ListNext               ToolHelp32Api::s_pfnHeap32ListNext              = NULL;
TfnHeap32First                  ToolHelp32Api::s_pfnHeap32First                 = NULL;
TfnHeap32Next                   ToolHelp32Api::s_pfnHeap32Next                  = NULL;
TfnToolhelp32ReadProcessMemory  ToolHelp32Api::s_pfnToolhelp32ReadProcessMemory = NULL;
TfnProcess32FirstW              ToolHelp32Api::s_pfnProcess32FirstW             = NULL;
TfnProcess32NextW               ToolHelp32Api::s_pfnProcess32NextW              = NULL;
TfnProcess32First               ToolHelp32Api::s_pfnProcess32First              = NULL;
TfnProcess32Next                ToolHelp32Api::s_pfnProcess32Next               = NULL;
TfnThread32First                ToolHelp32Api::s_pfnThread32First               = NULL;
TfnThread32Next                 ToolHelp32Api::s_pfnThread32Next                = NULL;
TfnModule32FirstW               ToolHelp32Api::s_pfnModule32FirstW              = NULL;
TfnModule32NextW                ToolHelp32Api::s_pfnModule32NextW               = NULL;
TfnModule32First                ToolHelp32Api::s_pfnModule32First               = NULL;
TfnModule32Next                 ToolHelp32Api::s_pfnModule32Next                = NULL;
bool                            ToolHelp32Api::s_bAutoUnload                    = true;

ToolHelp32Api::ToolHelp32Api()
{
}

// Remarks: We don't need any object of #ToolHelp32Api#.
//  We just need its static members.



bool ToolHelp32Api::ToolHelp32ApiExists()
{
    LoadAPI();
    return s_pfnCreateToolhelp32Snapshot != NULL;
}



void ToolHelp32Api::LoadAPI()
{
    if( s_hModule ) return;

    LockModuleWide lmw;

    if( s_hModule ) return;

    Nullify();

    s_hModule = ::LoadLibrary( _T(TOOLHELP32MODULE_NAME) );

    if( s_hModule )
    {
        s_pfnCreateToolhelp32Snapshot    = (TfnCreateToolhelp32Snapshot   ) GetProcAddress( s_hModule, "CreateToolhelp32Snapshot"    );
        s_pfnHeap32ListFirst             = (TfnHeap32ListFirst            ) GetProcAddress( s_hModule, "Heap32ListFirst"             );
        s_pfnHeap32ListNext              = (TfnHeap32ListNext             ) GetProcAddress( s_hModule, "Heap32ListNext"              );
        s_pfnHeap32First                 = (TfnHeap32First                ) GetProcAddress( s_hModule, "Heap32First"                 );
        s_pfnHeap32Next                  = (TfnHeap32Next                 ) GetProcAddress( s_hModule, "Heap32Next"                  );
        s_pfnToolhelp32ReadProcessMemory = (TfnToolhelp32ReadProcessMemory) GetProcAddress( s_hModule, "Toolhelp32ReadProcessMemory" );
        s_pfnProcess32FirstW             = (TfnProcess32FirstW            ) GetProcAddress( s_hModule, "Process32FirstW"             );
        s_pfnProcess32NextW              = (TfnProcess32NextW             ) GetProcAddress( s_hModule, "Process32NextW"              );
        s_pfnProcess32First              = (TfnProcess32First             ) GetProcAddress( s_hModule, "Process32First"              );
        s_pfnProcess32Next               = (TfnProcess32Next              ) GetProcAddress( s_hModule, "Process32Next"               );
        s_pfnThread32First               = (TfnThread32First              ) GetProcAddress( s_hModule, "Thread32First"               );
        s_pfnThread32Next                = (TfnThread32Next               ) GetProcAddress( s_hModule, "Thread32Next"                );
        s_pfnModule32FirstW              = (TfnModule32FirstW             ) GetProcAddress( s_hModule, "Module32FirstW"              );
        s_pfnModule32NextW               = (TfnModule32NextW              ) GetProcAddress( s_hModule, "Module32NextW"               );
        s_pfnModule32First               = (TfnModule32First              ) GetProcAddress( s_hModule, "Module32First"               );
        s_pfnModule32Next                = (TfnModule32Next               ) GetProcAddress( s_hModule, "Module32Next"                );
    }
}

void ToolHelp32Api::UnloadAPI()
{
    LockModuleWide lmw;

    if( s_hModule )
        ::FreeLibrary( s_hModule );

    Nullify();
}



void ToolHelp32Api::Nullify()
{
    s_pfnCreateToolhelp32Snapshot    = NULL;
    s_pfnHeap32ListFirst             = NULL;
    s_pfnHeap32ListNext              = NULL;
    s_pfnHeap32First                 = NULL;
    s_pfnHeap32Next                  = NULL;
    s_pfnToolhelp32ReadProcessMemory = NULL;
    s_pfnProcess32FirstW             = NULL;
    s_pfnProcess32NextW              = NULL;
    s_pfnProcess32First              = NULL;
    s_pfnProcess32Next               = NULL;
    s_pfnThread32First               = NULL;
    s_pfnThread32Next                = NULL;
    s_pfnModule32FirstW              = NULL;
    s_pfnModule32NextW               = NULL;
    s_pfnModule32First               = NULL;
    s_pfnModule32Next                = NULL;
    s_hModule = NULL;
}

#endif




ProcessesSnapshot::Info *ProcessesSnapshot::findByPid( DWORD pid )
{
    Info    *ptr   = getPtr();
    _size_t_ count = getCount();

    for( _idx_ i = 0 ; (_size_t_) i < count ; ++i, ++ptr )
        if( ptr->m_dwPid == pid )
            return ptr;

    return NULL;
}




ModulesSnapshot::Info *ModulesSnapshot::findByHModule( HMODULE hMod )
{
    Info    *ptr   = getPtr();
    _size_t_ count = getCount();

    for( _idx_ i = 0 ; (_size_t_) i < count ; ++i, ++ptr )
        if( ptr->m_hModule == hMod )
            return ptr;

    return NULL;
}




bool ProcessesModules::Init( bool bPsApi_preferred )
{
    bool bHasSomeApi = false;

    if( bPsApi_preferred )
    {
        if( m_bUsePsApi ) return true;
        m_bUseToolHelpApi = false;

        #ifndef _argon_no_psapi
        if( PsApiDll::PsApiExists() )
            bHasSomeApi = m_bUsePsApi = true;
        #endif

        #ifndef _argon_no_toolhelp32api
        if( !bHasSomeApi )
            if( ToolHelp32Api::ToolHelp32ApiExists() )
                bHasSomeApi = m_bUseToolHelpApi = true;
        #endif
    }
    else
    {
        if( m_bUseToolHelpApi ) return true;
        m_bUsePsApi = false;

        #ifndef _argon_no_toolhelp32api
        if( ToolHelp32Api::ToolHelp32ApiExists() )
            bHasSomeApi = m_bUseToolHelpApi = true;
        #endif

        #ifndef _argon_no_psapi
        if( !bHasSomeApi )
            if( PsApiDll::PsApiExists() )
                bHasSomeApi = m_bUsePsApi = true;
        #endif
    }

    return bHasSomeApi;
}



bool ProcessesModules::enumProcesses( ProcessesSnapshot& snapshot )
{
    if( m_bUsePsApi       ) return _enumProcessesPsApi      ( snapshot );
    if( m_bUseToolHelpApi ) return _enumProcessesToolHelpApi( snapshot );
    return false;
}

bool ProcessesModules::enumProcessModules( ModulesSnapshot& snapshot, DWORD pid, int maxModules )
{
    if( m_bUsePsApi       ) return _enumProcessModulesPsApi      ( snapshot, pid, maxModules );
    if( m_bUseToolHelpApi ) return _enumProcessModulesToolHelpApi( snapshot, pid, maxModules );
    return false;
}



bool ProcessesModules::_enumProcessesPsApi( ProcessesSnapshot& snapshot )
{
    #ifndef _argon_no_psapi

    snapshot.Empty();

    Array<DWORD,0> aPids;
    aPids.SetSize( 8192 );

    DWORD needed = 0;
    if( !argonEnumProcesses( aPids.Ptr(), (DWORD) aPids.GetByteSize(), &needed ) )
        return false;

    if( needed > (DWORD) aPids.GetByteSize() ) needed = (DWORD) aPids.GetByteSize();

    _size_t_ cProcesses = needed / sizeof(DWORD);

    snapshot.m_Snapshot.SetSize( cProcesses );

    DWORD *ppid = aPids.Ptr();
    for( _idx_ i = 0 ; (_size_t_) i < cProcesses ; ++i, ++ppid )
    {
        ProcessesSnapshot::Info *p = snapshot.m_Snapshot.PtrAt( i );
        if( !p ) return false;

        p->m_dwPid = *ppid;

        HANDLE hProcess = OpenProcess( PROCESS_QUERY_INFORMATION | PROCESS_VM_READ, FALSE, *ppid );
        if( hProcess )
        {
            // We get the first module in the process. This is then the executable file.
            HMODULE hMod = NULL;
            if( argonEnumProcessModules( hProcess, &hMod, sizeof(hMod), &needed ) )
                if( hMod )
                {
                    // #::GetModuleFileNameEx()# will *not* append a null-terminator
                    // if the passed string buffer is too short (otherwise it will).
                    argonGetModuleFileNameEx( hProcess, hMod, p->m_szFilepath, _countof(p->m_szFilepath) );
                    p->m_szFilepath[_countof(p->m_szFilepath) - 1] = (TCHAR) 0;  // this *is* necessary

                    argonGetModuleBaseName( hProcess, hMod, p->m_szName, _countof(p->m_szName) );
                    p->m_szName[_countof(p->m_szName) - 1] = (TCHAR) 0;  // this *is* necessary
                }

            CloseHandle( hProcess );
        }

        if( _isstrempty_(p->m_szName) )
            _strcpy_( p->m_szName, _filenameptrof_(p->m_szFilepath), _countof(p->m_szName) );

        if( _isstrempty_(p->m_szName) )
        {
            if( *ppid == 0 )
                _strcpy_( p->m_szName, _T("[System Idle Process]"), _countof(p->m_szName) );
            else if( *ppid == 4 )
                _strcpy_( p->m_szName, _T("System"), _countof(p->m_szName) );
            else
                _strcpy_( p->m_szName, m_pstrUnknownName, _countof(p->m_szName) );
        }
    }

    return true;

    #else

    return false;

    #endif
}

bool ProcessesModules::_enumProcessesToolHelpApi( ProcessesSnapshot& snapshot )
{
    #ifndef _argon_no_toolhelp32api

    snapshot.Empty();

    HANDLE hSnapshot = argonCreateToolhelp32Snapshot( TH32CS_SNAPPROCESS, 0 );
    if( hSnapshot == INVALID_HANDLE_VALUE ) return false;

    PROCESSENTRY32 pe;
    memset( &pe, 0, sizeof(pe) );
    pe.dwSize = sizeof(pe);

    if( argonProcess32First(hSnapshot, &pe) )
    {
        do
        {
            ProcessesSnapshot::Info *p = snapshot.m_Snapshot.AddNew();
            if( !p )
                { CloseHandle( hSnapshot ); return false; }

            p->m_dwPid = pe.th32ProcessID;
            _strcpy_( p->m_szName, _filenameptrof_(pe.szExeFile), _countof(p->m_szName) );
            // #pe.szExeFile# contains only the name without path (well
            // at least until WinXP, but who knows about future versions,
            // so for #p->m_szName# we use #_filenameptrof_()# in the above call).

            // Since #pe.szExeFile# contains only the name without path,
            // we need to get the first module (which is or should be the
            // exe's module) and from there, we can get the full path.
            _getProcessModuleFilepathToolHelpApi( p->m_szFilepath, pe.th32ProcessID, _countof(p->m_szFilepath) );

            if( pe.th32ProcessID != 0 && pe.th32ProcessID != 4 )    // PID 0 = idle process, 4 = System
                if( _isstrempty_(p->m_szFilepath) )
                    _strcpy_( p->m_szFilepath, pe.szExeFile, _countof(p->m_szFilepath) );

        } while( argonProcess32Next(hSnapshot, &pe) );
    }

    CloseHandle( hSnapshot );
    return true;

    #else

    return false;

    #endif
}

bool ProcessesModules::_enumProcessModulesPsApi( ModulesSnapshot& snapshot, DWORD pid, int maxModules )
{
    #ifndef _argon_no_psapi

    snapshot.Empty();

    if( maxModules == 0 ) return false;

    if( maxModules < 0 ) maxModules = 8192;  // use a large number if the user did not specify an upper limit

    Array<HMODULE> ahMods;
    ahMods.SetSize( maxModules );

    HANDLE hProcess = OpenProcess( PROCESS_QUERY_INFORMATION | PROCESS_VM_READ, FALSE, pid );
    if( !hProcess ) return false;

    DWORD needed = 0;
    if( !argonEnumProcessModules( hProcess, ahMods.Ptr(), (DWORD) ahMods.GetByteSize(), &needed ) )
        { CloseHandle( hProcess ); return false; }

    if( needed > (DWORD) ahMods.GetByteSize() ) needed = (DWORD) ahMods.GetByteSize();

    _size_t_ cModules = needed / sizeof(HMODULE);

    snapshot.m_Snapshot.SetSize( cModules );

    HMODULE *phMod = ahMods.Ptr();
    for( _idx_ i = 0 ; (_size_t_) i < cModules ; ++i, ++phMod )
    {
        ModulesSnapshot::Info *p = snapshot.m_Snapshot.PtrAt( i );
        if( !p )
            { CloseHandle( hProcess ); return false; }

        p->m_hModule = *phMod;

        MODULEINFO mi;
        argonGetModuleInformation( hProcess, *phMod, &mi, sizeof(mi) );

        p->m_pBaseAddr = mi.lpBaseOfDll;
        p->m_nBaseSize = mi.SizeOfImage;

        // #::GetModuleFileNameEx()# will *not* append a null-terminator
        // if the passed string buffer is too short (otherwise it will).
        argonGetModuleFileNameEx( hProcess, *phMod, p->m_szFilename, _countof(p->m_szFilename) );
        p->m_szFilename[_countof(p->m_szFilename) - 1] = (TCHAR) 0;  // this *is* necessary
    }

    CloseHandle( hProcess );
    return true;

    #else

    return false;

    #endif
}

bool ProcessesModules::_enumProcessModulesToolHelpApi( ModulesSnapshot& snapshot, DWORD pid, int maxModules )
{
    #ifndef _argon_no_toolhelp32api

    snapshot.Empty();

    if( maxModules == 0 ) return false;

    if( pid == 0 ) return false;
    // We must check PID for zero, because #CreateToolhelp32Snapshot()# interprets
    // a PID of zero as the current process.

    HANDLE hSnapshot = argonCreateToolhelp32Snapshot( TH32CS_SNAPMODULE, pid );
    if( hSnapshot == INVALID_HANDLE_VALUE ) return false;

    MODULEENTRY32 me;
    memset( &me, 0, sizeof(me) );
    me.dwSize = sizeof(me);

    if( argonModule32First(hSnapshot, &me) )
    {
        do
        {
            ModulesSnapshot::Info *p = snapshot.m_Snapshot.AddNew();
            if( !p )
                { CloseHandle( hSnapshot ); return false; }

            p->m_hModule   = me.hModule;
            p->m_pBaseAddr = me.modBaseAddr;
            p->m_nBaseSize = me.modBaseSize;
            _strcpy_( p->m_szFilename, me.szExePath, _countof(p->m_szFilename) );

            if( --maxModules == 0 )
                break;

        } while( argonModule32Next(hSnapshot, &me) );
    }

    CloseHandle( hSnapshot );
    return true;

    #else

    return false;

    #endif
}

bool ProcessesModules::_getProcessModuleFilepathToolHelpApi( PTSTR pszFilepath, DWORD pid, _size_t_ cFilepath )
{
    #ifndef _argon_no_toolhelp32api

    if( !pszFilepath || cFilepath <= 0 ) return false;

    if( pid == 0 ) return false;
    // We must check PID for zero, because #CreateToolhelp32Snapshot()# interprets
    // a PID of zero as the current process.

    HANDLE hSnapshot = argonCreateToolhelp32Snapshot( TH32CS_SNAPMODULE, pid );
    if( hSnapshot == INVALID_HANDLE_VALUE ) return false;

    MODULEENTRY32 me;
    memset( &me, 0, sizeof(me) );
    me.dwSize = sizeof(me);

    if( !argonModule32First(hSnapshot, &me) )
        { CloseHandle( hSnapshot ); return false; }

    _strcpy_( pszFilepath, me.szExePath, cFilepath );

    CloseHandle( hSnapshot );
    return true;

    #else

    return false;

    #endif
}




}
